@@ -2,17 +2,22 @@ import typing
 import datetime
 
 from .emoji import Emoji
+from .extras import BYTES_RESPONSE
 from .permission import Role, PermissionFlags
 from .snowflake import Snowflake
 from .stage import StageInstance
 from .sticker import Sticker
 from .user import User
 from ..utils import cdn_url
-from ..base.model import DiscordObjectBase, TypeBase, FlagBase
+from ..base.model import DiscordObjectBase, TypeBase, FlagBase, AbstractObject
 from ..base.http import EmptyObject
 
 if typing.TYPE_CHECKING:
     from .channel import Channel
+    from .invite import Invite
+    from .voice import VoiceRegion
+    from ..api import APIClient
+    from ..cache import GuildCacheContainer
 
 
 class Guild(DiscordObjectBase):
@@ -21,7 +26,7 @@ class Guild(DiscordObjectBase):
     RESPONSE_AS_LIST = typing.Union[typing.List["Guild"], typing.Awaitable[typing.List["Guild"]]]
     _cache_type = "guild"
 
-    def __init__(self, client, resp):
+    def __init__(self, client: "APIClient", resp: dict):
         from .channel import Channel  # Prevent circular import.
         from .event import PresenceUpdate
         from .voice import VoiceState
@@ -78,21 +83,21 @@ class Guild(DiscordObjectBase):
         self.stage_instances: typing.Optional[typing.List[StageInstance]] = [StageInstance.create(client, x) for x in resp.get("stage_instances", [])]
         self.stickers: typing.Optional[typing.List[Sticker]] = [Sticker.create(client, x) for x in resp.get("stickers", [])]
 
-        self.cache = client.cache.get_guild_container(self.id) if client.has_cache else None
+        self.cache: typing.Optional[GuildCacheContainer] = client.cache.get_guild_container(self.id) if client.has_cache else None
 
-    def icon_url(self, *, extension="webp", size=1024) -> typing.Optional[str]:
+    def icon_url(self, *, extension: str = "webp", size: int = 1024) -> typing.Optional[str]:
         if self.icon:
             return cdn_url("icons/{guild_id}", image_hash=self.icon, extension=extension, size=size, guild_id=self.id)
 
-    def splash_url(self, *, extension="webp", size=1024) -> typing.Optional[str]:
+    def splash_url(self, *, extension: str = "webp", size: int = 1024) -> typing.Optional[str]:
         if self.splash:
             return cdn_url("splashes/{guild_id}", image_hash=self.splash, extension=extension, size=size, guild_id=self.id)
 
-    def discovery_splash_url(self, *, extension="webp", size=1024) -> typing.Optional[str]:
+    def discovery_splash_url(self, *, extension: str = "webp", size: int = 1024) -> typing.Optional[str]:
         if self.discovery_splash:
             return cdn_url("discovery-splashes/{guild_id}", image_hash=self.discovery_splash, extension=extension, size=size, guild_id=self.id)
 
-    def banner_url(self, *, extension="webp", size=1024) -> typing.Optional[str]:
+    def banner_url(self, *, extension: str = "webp", size: int = 1024) -> typing.Optional[str]:
         if self.banner:
             return cdn_url("banners/{guild_id}", image_hash=self.banner, extension=extension, size=size, guild_id=self.id)
 
@@ -102,14 +107,14 @@ class Guild(DiscordObjectBase):
     def delete(self):
         return self.client.delete_guild(self)
 
-    def modify(self, **kwargs):
+    def modify(self, **kwargs) -> "Guild.RESPONSE":
         return self.client.modify_guild(self, **kwargs)
 
     @property
     def edit(self):
         return self.modify
 
-    def request_channels(self) -> typing.Union[typing.List["Channel"], typing.Awaitable[typing.List["Channel"]]]:
+    def request_channels(self) -> "Channel.RESPONSE_AS_LIST":
         return self.client.request_guild_channels(self)
 
     def create_channel(self, name: str, **kwargs) -> "Channel.RESPONSE":
@@ -118,13 +123,13 @@ class Guild(DiscordObjectBase):
     def modify_channel_positions(self, *params: dict, reason: str = None):
         return self.client.modify_guild_channel_positions(self, *params, reason=reason)
 
-    def list_active_threads(self) -> typing.Union[typing.List["Channel"], typing.Awaitable[typing.List["Channel"]]]:
+    def list_active_threads(self) -> "Channel.RESPONSE_AS_LIST":
         return self.client.list_active_threads_as_guild(self)
 
     def request_member(self, user: User.TYPING) -> "GuildMember.RESPONSE":
         return self.client.request_guild_member(self, user)
 
-    def list_members(self, limit: int = None, after: str = None) -> typing.Union[typing.List["GuildMember"], typing.Awaitable[typing.List["GuildMember"]]]:
+    def list_members(self, limit: int = None, after: str = None) -> "GuildMember.RESPONSE_AS_LIST":
         return self.client.list_guild_members(self, limit, after)
 
     def remove_guild_member(self, user: User.TYPING):
@@ -148,7 +153,7 @@ class Guild(DiscordObjectBase):
     def unban(self):
         return self.remove_ban
 
-    def request_roles(self) -> typing.List[Role]:
+    def request_roles(self) -> Role.RESPONSE_AS_LIST:
         return self.client.request_guild_roles(self)
 
     def create_role(self,
@@ -158,7 +163,7 @@ class Guild(DiscordObjectBase):
                     color: int = None,
                     hoist: bool = None,
                     mentionable: bool = None,
-                    reason: str = None) -> Role:
+                    reason: str = None) -> Role.RESPONSE:
         kwargs = {"name": name,
                   "permissions": permissions,
                   "color": color,
@@ -167,7 +172,7 @@ class Guild(DiscordObjectBase):
                   "reason": reason}
         return self.client.create_guild_role(self, **kwargs)
 
-    def modify_role_positions(self, *params: dict, reason: str = None):
+    def modify_role_positions(self, *params: dict, reason: str = None) -> Role.RESPONSE_AS_LIST:
         return self.client.modify_guild_role_positions(self, *params, reason=reason)
 
     def modify_role(self,
@@ -178,7 +183,7 @@ class Guild(DiscordObjectBase):
                     color: int = EmptyObject,
                     hoist: bool = EmptyObject,
                     mentionable: bool = EmptyObject,
-                    reason: str = None):
+                    reason: str = None) -> Role.RESPONSE:
         kwargs = {"role": role,
                   "name": name,
                   "permissions": permissions,
@@ -191,7 +196,7 @@ class Guild(DiscordObjectBase):
     def delete_role(self, role: Role.TYPING, *, reason: str = None):
         return self.client.delete_guild_role(self, role, reason=reason)
 
-    def request_prune_count(self, *, days: int = None, include_roles: typing.List[Role.TYPING] = None):
+    def request_prune_count(self, *, days: int = None, include_roles: typing.List[Role.TYPING] = None) -> AbstractObject.RESPONSE:
         return self.client.request_guild_prune_count(self, days=days, include_roles=include_roles)
 
     def begin_prune(self,
@@ -199,41 +204,41 @@ class Guild(DiscordObjectBase):
                     days: int = 7,
                     compute_prune_count: bool = True,
                     include_roles: typing.List[Role.TYPING] = None,
-                    reason: str = None):
+                    reason: str = None) -> AbstractObject.RESPONSE:
         return self.client.begin_guild_prune(self, days=days, compute_prune_count=compute_prune_count, include_roles=include_roles, reason=reason)
 
     @property
     def prune(self):
         return self.begin_prune
 
-    def request_voice_regions(self):
+    def request_voice_regions(self) -> "VoiceRegion.RESPONSE_AS_LIST":
         return self.client.request_guild_voice_regions(self)
 
-    def request_invites(self):
+    def request_invites(self) -> "Invite.RESPONSE_AS_LIST":
         return self.client.request_guild_invites(self)
 
-    def request_integrations(self):
+    def request_integrations(self) -> "Integration.RESPONSE_AS_LIST":
         return self.client.request_guild_integrations(self)
 
     def delete_integration(self, integration: "Integration.TYPING", *, reason: str = None):
         return self.client.delete_guild_integration(self, integration, reason=reason)
 
-    def request_widget_settings(self):
+    def request_widget_settings(self) -> "GuildWidget.RESPONSE":
         return self.client.request_guild_widget_settings(self)
 
-    def modify_widget(self, *, enabled: bool = None, channel: "Channel.TYPING" = EmptyObject, reason: str = None):
+    def modify_widget(self, *, enabled: bool = None, channel: "Channel.TYPING" = EmptyObject, reason: str = None) -> "GuildWidget.RESPONSE":
         return self.client.modify_guild_widget(self, enabled=enabled, channel=channel, reason=reason)
 
-    def request_widget(self):
+    def request_widget(self) -> AbstractObject.RESPONSE:
         return self.client.request_guild_widget(self)
 
-    def request_vanity_url(self):
+    def request_vanity_url(self) -> AbstractObject.RESPONSE:
         return self.client.request_guild_vanity_url(self)
 
-    def request_widget_image(self, style: str = None):
+    def request_widget_image(self, style: str = None) -> BYTES_RESPONSE:
         return self.client.request_guild_widget_image(self, style)
 
-    def request_welcome_screen(self):
+    def request_welcome_screen(self) -> "WelcomeScreen.RESPONSE":
         return self.client.request_guild_welcome_screen(self)
 
     def modify_guild_welcome_screen(self,
@@ -241,7 +246,7 @@ class Guild(DiscordObjectBase):
                                     enabled: bool = EmptyObject,
                                     welcome_channels: typing.List[typing.Union["WelcomeScreenChannel", dict]] = EmptyObject,
                                     description: str = EmptyObject,
-                                    reason: str = None):
+                                    reason: str = None) -> "WelcomeScreen.RESPONSE":
         return self.client.modify_guild_welcome_screen(self, enabled=enabled, welcome_channels=welcome_channels, description=description, reason=reason)
 
     def modify_user_voice_state(self,
@@ -256,12 +261,12 @@ class Guild(DiscordObjectBase):
         return self.client.leave_guild(self)
 
     @property
-    def get(self):
+    def get(self) -> typing.Optional["GuildCacheContainer.get"]:
         """Alias of ``Guild.cache.get``."""
         if self.cache:
             return self.cache.get
 
-    def get_owner(self):
+    def get_owner(self) -> typing.Optional[typing.Union["GuildMember", User]]:
         if self.cache:
             return self.get(self.owner_id, "member") or self.get(self.owner_id, "user")
 
@@ -313,26 +318,26 @@ class SystemChannelFlags(FlagBase):
 class GuildPreview:
     RESPONSE = typing.Union["GuildPreview", typing.Awaitable["GuildPreview"]]
 
-    def __init__(self, client, resp):
-        self.id = Snowflake(resp["id"])
-        self.name = resp["name"]
-        self.icon = resp["icon"]
-        self.splash = resp["splash"]
-        self.discovery_splash = resp["discovery_splash"]
-        self.emojis = [Emoji(client, x) for x in resp["emojis"]]
-        self.features = resp["features"]
-        self.approximate_member_count = resp["approximate_member_count"]
-        self.approximate_presence_count = resp["approximate_presence_count"]
-        self.description = resp["description"]
+    def __init__(self, client: "APIClient", resp: dict):
+        self.id: Snowflake = Snowflake(resp["id"])
+        self.name: str = resp["name"]
+        self.icon: typing.Optional[str] = resp["icon"]
+        self.splash: typing.Optional[str] = resp["splash"]
+        self.discovery_splash: typing.Optional[str] = resp["discovery_splash"]
+        self.emojis: typing.List[Emoji] = [Emoji(client, x) for x in resp["emojis"]]
+        self.features: typing.List[str] = resp["features"]
+        self.approximate_member_count: int = resp["approximate_member_count"]
+        self.approximate_presence_count: int = resp["approximate_presence_count"]
+        self.description: typing.Optional[str] = resp["description"]
 
 
 class GuildWidget:
     RESPONSE = typing.Union["GuildWidget", typing.Awaitable["GuildWidget"]]
     RESPONSE_AS_LIST = typing.Union[typing.List["GuildWidget"], typing.Awaitable[typing.List["GuildWidget"]]]
 
-    def __init__(self, resp):
-        self.enabled = resp["enabled"]
-        self.channel_id = Snowflake.optional(resp["channel_id"])
+    def __init__(self, resp: dict):
+        self.enabled: bool = resp["enabled"]
+        self.channel_id: typing.Optional[Snowflake] = Snowflake.optional(resp["channel_id"])
 
 
 class GuildMember:
@@ -340,29 +345,30 @@ class GuildMember:
     RESPONSE = typing.Union["GuildMember", typing.Awaitable["GuildMember"]]
     RESPONSE_AS_LIST = typing.Union[typing.List["GuildMember"], typing.Awaitable[typing.List["GuildMember"]]]
 
-    def __init__(self, client, resp, *, user: User = None, guild_id=None):
-        self.raw = resp
-        self.client = client
+    def __init__(self, client: "APIClient", resp: dict, *, user: User = None, guild_id: Snowflake = None):
+        self.raw: dict = resp
+        self.client: "APIClient" = client
         self.user = user
         self.__user = resp.get("user")
-        self.user = User.create(client, self.__user) if self.__user and not self.user else self.user if self.user else self.__user
-        self.nick = resp.get("nick")
-        self.roles = [client.get(x, "role") for x in resp["roles"]] if client.has_cache else [Snowflake(x) for x in resp["roles"]]
-        self.joined_at = datetime.datetime.fromisoformat(resp["joined_at"])
+        self.user: typing.Optional[User] = User.create(client, self.__user) if self.__user and not self.user else self.user if self.user else self.__user
+        self.nick: typing.Optional[str] = resp.get("nick")
+        self.roles: typing.Optional[typing.List[Role]] = [client.get(x, "role") for x in resp["roles"]] if client.has_cache else []
+        self.role_ids: typing.List[Snowflake] = [Snowflake(x) for x in resp["roles"]]
+        self.joined_at: datetime.datetime = datetime.datetime.fromisoformat(resp["joined_at"])
         self.__premium_since = resp.get("premium_since")
-        self.premium_since = datetime.datetime.fromisoformat(self.__premium_since) if self.__premium_since else self.__premium_since
-        self.deaf = resp.get("deaf", False)
-        self.mute = resp.get("mute", False)
-        self.pending = resp.get("pending", False)
+        self.premium_since: datetime.datetime = datetime.datetime.fromisoformat(self.__premium_since) if self.__premium_since else self.__premium_since
+        self.deaf: typing.Optional[bool] = resp.get("deaf", False)
+        self.mute: typing.Optional[bool] = resp.get("mute", False)
+        self.pending: typing.Optional[bool] = resp.get("pending", False)
         self.__permissions = resp.get("permissions")
-        self.guild_id = Snowflake.optional(resp.get("guild_id")) or Snowflake.ensure_snowflake(guild_id)
+        self.guild_id: typing.Optional[Snowflake] = Snowflake.optional(resp.get("guild_id")) or Snowflake.ensure_snowflake(guild_id)
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self.nick or (self.user.username if self.user else None)
 
-    def __int__(self):
+    def __int__(self) -> int:
         if self.user:
-            return self.user.id
+            return int(self.user.id)
 
     def remove(self):
         return self.client.remove_guild_member(self.guild_id, self)
@@ -375,7 +381,7 @@ class GuildMember:
         return self.client.create_guild_ban(self.guild_id, self, delete_message_days=delete_message_days, reason=reason)
 
     @property
-    def mention(self):
+    def mention(self) -> str:
         if self.user:
             return f"<@!{self.user.id}>"
 
@@ -395,7 +401,7 @@ class GuildMember:
             return PermissionFlags.from_value(0)
 
     @classmethod
-    def create(cls, client, resp, *, user=None, guild_id=None, cache: bool = True):
+    def create(cls, client: "APIClient", resp: dict, *, user: User = None, guild_id: Snowflake.TYPING = None, cache: bool = True):
         if cache and client.has_cache and (guild_id or resp.get("guild_id")) and (user or resp.get("user")):
             _guild_id = guild_id or resp.get("guild_id")
             _user_id = user.id if isinstance(user, User) else resp["user"]["id"]
@@ -418,34 +424,34 @@ class Integration:
     RESPONSE = typing.Union["Integration", typing.Awaitable["Integration"]]
     RESPONSE_AS_LIST = typing.Union[typing.List["Integration"], typing.Awaitable[typing.List["Integration"]]]
 
-    def __init__(self, client, resp):
-        self.client = client
-        self.raw = resp
-        self.id = Snowflake(resp["id"])
-        self.name = resp["name"]
-        self.type = resp["type"]
-        self.enabled = resp["enabled"]
-        self.syncing = resp.get("syncing")
-        self.role_id = Snowflake.optional(resp.get("role_id"))
-        self.enable_emoticons = resp.get("enable_emoticons")
+    def __init__(self, client: "APIClient", resp: dict):
+        self.client: "APIClient" = client
+        self.raw: dict = resp
+        self.id: Snowflake = Snowflake(resp["id"])
+        self.name: str = resp["name"]
+        self.type: str = resp["type"]
+        self.enabled: typing.Optional[bool] = resp.get("enabled")
+        self.syncing: typing.Optional[bool] = resp.get("syncing")
+        self.role_id: typing.Optional[Snowflake] = Snowflake.optional(resp.get("role_id"))
+        self.enable_emoticons: typing.Optional[bool] = resp.get("enable_emoticons")
         self.__expire_behavior = resp.get("expire_behavior")
-        self.expire_behavior = IntegrationExpireBehaviors(int(self.__expire_behavior)) if self.__expire_behavior else self.__expire_behavior
-        self.expire_grace_period = resp.get("expire_grace_period")
+        self.expire_behavior: typing.Optional[IntegrationExpireBehaviors] = IntegrationExpireBehaviors(int(self.__expire_behavior)) if self.__expire_behavior else self.__expire_behavior
+        self.expire_grace_period: typing.Optional[int] = resp.get("expire_grace_period")
         self.__user = resp.get("user")
-        self.user = User.create(client, self.__user) if self.__user else self.__user
-        self.account = IntegrationAccount(resp["account"])
+        self.user: typing.Optional[User] = User.create(client, self.__user) if self.__user else self.__user
+        self.account: IntegrationAccount = IntegrationAccount(resp["account"])
         self.__synced_at = resp.get("synced_at")
-        self.synced_at = datetime.datetime.fromisoformat(self.__synced_at) if self.__synced_at else self.__synced_at
-        self.subscriber_count = resp.get("subscriber_count")
-        self.revoked = resp.get("revoked")
+        self.synced_at: typing.Optional[datetime.datetime] = datetime.datetime.fromisoformat(self.__synced_at) if self.__synced_at else self.__synced_at
+        self.subscriber_count: typing.Optional[int] = resp.get("subscriber_count")
+        self.revoked: typing.Optional[bool] = resp.get("revoked")
         self.__application = resp.get("application")
-        self.application = IntegrationApplication(client, self.__application) if self.__application else self.__application
+        self.application: typing.Optional[IntegrationApplication] = IntegrationApplication(client, self.__application) if self.__application else self.__application
 
-    def __int__(self):
+    def __int__(self) -> int:
         return int(self.id)
 
     @property
-    def role(self):
+    def role(self) -> typing.Optional[Role]:
         if self.client.has_cache:
             return self.client.cache.get(self.role_id, "role")
 
@@ -456,22 +462,22 @@ class IntegrationExpireBehaviors(TypeBase):
 
 
 class IntegrationAccount:
-    def __init__(self, resp):
-        self.id = resp["id"]
-        self.name = resp["name"]
+    def __init__(self, resp: dict):
+        self.id: str = resp["id"]
+        self.name: str = resp["name"]
 
 
 class IntegrationApplication:
-    def __init__(self, client, resp):
-        self.id = Snowflake(resp["id"])
-        self.name = resp["name"]
-        self.icon = resp["icon"]
-        self.description = resp["description"]
-        self.summary = resp["summary"]
+    def __init__(self, client: "APIClient", resp: dict):
+        self.id: Snowflake = Snowflake(resp["id"])
+        self.name: str = resp["name"]
+        self.icon: typing.Optional[str] = resp["icon"]
+        self.description: str = resp["description"]
+        self.summary: str = resp["summary"]
         self.__bot = resp.get("bot")
-        self.bot = User.create(client, self.__bot) if self.__bot else self.__bot
+        self.bot: typing.Optional[User] = User.create(client, self.__bot) if self.__bot else self.__bot
 
-    def icon_url(self, *, extension="webp", size=1024):
+    def icon_url(self, *, extension: str = "webp", size: int = 1024) -> typing.Optional[str]:
         return cdn_url("app-icons/{application_id}", image_hash=self.icon, extension=extension, size=size, application_id=self.id)
 
 
@@ -479,29 +485,29 @@ class Ban:
     RESPONSE = typing.Union["Ban", typing.Awaitable["Ban"]]
     RESPONSE_AS_LIST = typing.Union[typing.List["Ban"], typing.Awaitable[typing.List["Ban"]]]
 
-    def __init__(self, client, resp):
-        self.reason = resp["reason"]
-        self.user = User.create(client, resp["user"])
+    def __init__(self, client: "APIClient", resp: dict):
+        self.reason: typing.Optional[str] = resp["reason"]
+        self.user: User = User.create(client, resp["user"])
 
 
 class WelcomeScreen:
     RESPONSE = typing.Union["WelcomeScreen", typing.Awaitable["WelcomeScreen"]]
     RESPONSE_AS_LIST = typing.Union[typing.List["WelcomeScreen"], typing.Awaitable[typing.List["WelcomeScreen"]]]
 
-    def __init__(self, resp):
-        self.description = resp["description"]
-        self.welcome_channels = [WelcomeScreenChannel(x) for x in resp["welcome_channels"]]
+    def __init__(self, resp: dict):
+        self.description: typing.Optional[str] = resp["description"]
+        self.welcome_channels: typing.List[WelcomeScreenChannel] = [WelcomeScreenChannel(x) for x in resp["welcome_channels"]]
 
     def to_dict(self):
         return {"description": self.description, "welcome_channels": [x.to_dict() for x in self.welcome_channels]}
 
 
 class WelcomeScreenChannel:
-    def __init__(self, resp):
-        self.channel_id = Snowflake(resp["channel_id"])
-        self.description = resp["description"]
-        self.emoji_id = Snowflake.optional(resp["emoji_id"])
-        self.emoji_name = resp["emoji_name"]
+    def __init__(self, resp: dict):
+        self.channel_id: Snowflake = Snowflake(resp["channel_id"])
+        self.description: str = resp["description"]
+        self.emoji_id: typing.Optional[Snowflake] = Snowflake.optional(resp["emoji_id"])
+        self.emoji_name: typing.Optional[str] = resp["emoji_name"]
 
-    def to_dict(self):
+    def to_dict(self) -> dict:
         return {"channel_id": str(self.channel_id), "description": self.description, "emoji_id": str(self.emoji_id), "emoji_name": self.emoji_name}
